<?php
/**
 * plugins/abook_imsp/abook_class.php
 *
 * $Id: abook_class.php,v 1.2 2004/09/01 10:56:01 veg Exp $
 */
require("imsp/cIMSPABook.php");

define(SEARCH_WILDCARD,1);
define(SEARCH_EXACT,2);

/**
 * imsp address book backend class
 */
class abook_imsp extends addressbook_backend {
    var $btype = 'local';
    var $bname = 'imsp';
    
    var $writeable = true;
	var $imsp;
	var $errstr="";
	var $errno=0;
	var $abookname="";
      
    /* ========================== Private ======================= */
      
    /* Constructor */
    function abook_imsp($param) {
        $this->sname = _("New address book");
         
        if (is_array($param)) {
            if (!empty($param['name'])) {
               $this->sname = $param['name'];
            }

            if (isset($param['writeable'])) {
               $this->writeable = $param['writeable'];
            }

            if (isset($param['listing'])) {
               $this->listing = $param['listing'];
            }

            $this->open(true);
        }
        else {
            return $this->set_error('Invalid argument to constructor');
        }
    }

    /**
     *
     */
    function open() {
	global $onetimepad,$username,$key;

	$password = OneTimePadDecrypt($key, $onetimepad);
	if (! $this->imsp = new IMSPAddressBook()) {
       	    $this->errno = -1;
            $this->errstr = "Can't create IMSP object";
            return false;
        }

	$this->imsp->escape_special_characters = 0;
                                                                                
        if (! $this->imsp->login($username, $password)) {
            $this->errno = -1;
            $this->errstr = "Can't login";
            return false;
		}
		$this->abookname=$username;

    	return true;
    }

    /**
     *
     */
    function close() {
		$this->imsp->logout();
    }
      
      
    /* ========================== Public ======================== */

    /**
     * Search address function
     * @param expr string search expression
     * @return either false, or an array of address arrays
     */
    function search($expr,$searchtype=SEARCH_WILDCARD) {
        if (empty($expr)) {
			return false;
        }

	$ret=array();
	$alias = $searchtype==SEARCH_WILDCARD ? '*'.strtolower($expr).'*' : strtolower($expr);

	$entries = $this->imsp->searchAddressBook($this->abookname, $alias, "alias");                                                                                
	if (count($entries)<=0) {
		return(false);
	}
	for($i = 0 ; $i < count($entries) ; $i++) {
        $entry = $this->imsp->getAddressBookEntry($this->abookname,
                                                             $entries[$i]);
	if (preg_match('/^\s*(\w+)\s*(.*)/',$entry['name'],$namematches)) {
		$firstname=$namematches[1];
		$lastname=$namematches[2];
	} else {
		$firstname=$entry['name'];
		$lastname="";
	}
	if ($entries[$i]) {
		array_push($ret,array('nickname' => $entry['alias'],
				'name' => $entries[$i],
				'firstname' => $firstname,
				'lastname' => $lastname,
				'email' => $entry['email'],
				'label' => "info",
				'backend' => $this->bnum,
				'source' => $this->sname));
		}
        }

        return $ret;
    }
     
    /**
     * Lookup alias
     * @param alias string
     */
    function lookup($alias) {
	$ret=$this->search($alias,SEARCH_EXACT);
        return $ret[0];
    }

    /**
     * List all addresses
     * @return array
     */
    function list_addr() {
	$ret=$this->search('*',SEARCH_WILDCARD);
        return $ret;
    }

    /**
     * Add address
     * @param userdata
     * @return boolean
     */
    function add($userdata) {
        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

        /* See if user exist already */
        $ret = $this->lookup($userdata['nickname']);
        if (!empty($ret)) {
            return $this->set_error(sprintf(_("User '%s' already exist"),
                                            $ret['nickname']));
        }

	$nickname=trim($userdata['nickname']);
	$name=$userdata['firstname'] ? $userdata['firstname']." " : "";
	$name.=$userdata['lastname'];
	if (! $name) {
		$name=$nickname;
	}
	$name=trim($name);
	$newentry=array('name' => $name,
			'email' => $userdata['email'],
			'alias' => $nickname,
			'info' => $userdata['label'],
			'firstname' => $userdata['nickname']);
	$this->imsp->addAddress($this->abookname, $newentry);

	return true;
        return $this->set_error(_("Address add operation failed"));
    }

    /**
     * Delete address
     * @param alias
     * @return boolean
     */
    function remove($alias) {
        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

	foreach($alias as $k => $nickname) {
		if ($entry=$this->lookup($nickname)) {
			$this->imsp->deleteAddress($this->abookname, $entry['name']);
		}
	}

	return true;
	// Return error message if operation fails
	// Not worth the hassle...veg
        return $this->set_error(_("Address delete operation failed"));
    }

    /**
     * Modify address
     * @param alias
     * @param userdata
     * @return boolean
     */
    function modify($alias, $userdata) {
        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

         /* See if user exist */
        $ret = $this->lookup($alias);
        if (empty($ret)) {
            return $this->set_error(sprintf(_("User '%s' does not exist"),
                                            $alias));
        }

        $this->remove(array($alias));
        $this->add($userdata);

	// FIXME:
	// return true if operation is succesful.
	return true;
	// Return error message if operation fails
        return $this->set_error(_("Address modify operation failed"));
    }
} /* End of class abook_imsp */
?>
